function r=DMA(grocer_namey,grocer_exof,grocer_exonf,grocer_lamda,grocer_alpha,varargin)
 
// PURPOSE: Prediction using DMA
//---------------------------------------------------------
// INPUT:
// * grocer_namey = a time series, a real (n x 1) vector or a
// string equal to the name of a time series or a (n x 1) real
// vector between quotes
// * grocer_exof = a (nobs x nz) vector of compulsory
//   exogenous variables
// * grocer_exonf = a (nobs x nx) vector of potential
//   exogenous variables
// * grocer_lamda = a scalar, the forgetting factor for past
//   observations in the calculation of the time-varying
//   coefficients
// * grocer_alpha = a scalar, the forgetting factor of past
//   performance (a forecast k periods old have a weight
//   equal to k^alpha)
// * varargin: optional arguments that can be:
//   - 'prefix_db=xx' where xx is the prefix (folder and
//   start of the file) of the database where data with
//   a big volume (probabilities and coefficient updates,
//   forecast of each model)
//   - 'dropna=%t' if the user wants to work with all dates
//   with non NA values
//   - the string 'noprint' if the user doesn't want to
//    print the results of the regression
//   - 'nxmax=xx' where xx is the the maximum # of variables
//     allowed in a model (default: no constraint)
//   -  'PRM=xx' where xx is either a scalar or a (N x 1)
//     vectors of a priori values for the coefficients
//   -  'PRV=' thewhere xx is either a scalar or a (N x 1)
//     vectors of a priori values for their variance
//---------------------------------------------------------
// OUTPUT:
// r = a results tlist with:
// - r('meth') = 'DMA'
// - r('y') = (nobs x 1) vector of endogenous variables
// - r('x fixed') = (nobs x nZ) vector of compulsory
//   exogenous variables
// - r('x variable') = (nobs x nZ) vector of poten_tial
//   exogenous variables
// - r('lamda') = a scalar, the forgetting factor for past
//   observations in the calculation of the time-varying
//   coefficients
// - r('alpha') = a scalar, the forgetting factor of past
//   performance (a forecast k periods old have a weight
//   equal to k^alpha)
// - r('nx max') = the maximum # of non zero coefficients in
//   any model
// - r('models proba') = a (K x nobs) giving the
//   probability assigned to each model at each date
// - r('vars proba') = a (nx x nobs) giving the
//   probability assigned to each variable at each date
// - r('models forecast') = a (K x nobs) giving the
//   forecast generated by each model at each date
// - r('global forecast') = a (nx x nobs) giving the
//   weighted forecast of all possible models
// - r('best forecast') = a (1 x nobs) giving the
//   forecast by the best model at each date
// - r('combinations') = a (K x (nZ+nx)) matrix
//   indicating for each model which variables are
//   in the model (1 value in corresponding column)
//   and which variables are not in the model (0 value
//   in corresponding column)
// - r('library') = name of the library where big volume
//   results are stored
// - r('n db') = # of corresponding databases
// - r('exp # of models') = (nobs x 1) vector of # of expected
//   models at each date
// - r('exp theta') = (nobs x (N+nZ)) vector of expected
//   values at each date for each variable
// - r('namey') = name of the y variable
// - r('fixed namex') = name of the compulsory
//   exogenous variables
// - r('variable namex') = name of the potential
//   exogenous variables
// - r('dropna') = boolean indicating if NAs have been dropped
// - r('prests') = boolean indicating the presence or
//     absence of a time series in the regression
// - r('bounds') = if there is a time series in the
//     regression, the bounds of the regression
//---------------------------------------------------------
// REFERENCES:
// Koop and Korobilis, 2009, Forecasting
// Inflation using Dynamic Model Averaging
// http:www.rcfea.org/RePEc/pdf/wp34_09.pdf
//---------------------------------------------------------
// Copyright E. Dubois (2012)
// http://grocer.toolbox.free.fr/grocer.html
// translated and substantially adapted from a matlab
// programm by Dimitris Korobilis, University of Strathclyde
 
global GROCERDIR
// set defaults
grocer_dropna=%f
grocer_prt=%f
grocer_prefix_db=GROCERDIR+'\temp\db'
 
// read variable inputs
grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
    grocer_argi=strsubst(varargin(grocer_i),' ','')
    if part(grocer_argi,1:7) == 'nxmax=' | ...
       part(grocer_argi,1:4) == 'PRM=' | ...
       part(grocer_argi,1:4) == 'PRV=' then
 
       execstr('grocer_'+varargin(grocer_i))
 
   elseif grocer_argi == 'dropna' then
      grocer_dropna=%t
 
   elseif part(grocer_argi,1:10) == 'prefix_db=' then
    // true index that takes into account ptential blanks
      grocer_indeq=strindex(varargin(grocer_i),'=')
      grocer_prefix_db=part(varargin(grocer_i),grocer_indeq(1)+1:length(varargin(grocer_i)))
 
    elseif grocer_argi == 'noprint' then
       grocer_prt=%f
 
    end
end
 
// recover matrices, names, bounds,...
[grocer_mats,grocer_names,grocer_prests,grocer_b,grocer_nonna]=...
   explon(list( grocer_namey,grocer_exof,grocer_exonf),['endogenous' 'fixed exogenous' 'variable exogenous'],[],%t,grocer_dropna)
 
y=grocer_mats(1)
zf=grocer_mats(2)
zv=grocer_mats(3)
N=size(zv,2)
nZ=size(zf,2)
 
if ~exists('grocer_nxmax','local') then
// set the maximum # of variables to the max
   grocer_nxmax=N
end
if ~exists('grocer_PRM','local') then
// default prior for the mean of the coefficients
   grocer_PRM=zeros(1,N+nZ)
end
if ~exists('grocer_PRV','local') then
// default prior for the variance of the coefficients
   grocer_PRV=100*ones(1,N+nZ)
elseif min(size(grocer_PRV)) == 1 then
   grocer_PRV=diag(grocer_PRV)
end
 
r=DMA1(y,zf,zv,grocer_lamda,grocer_alpha,grocer_nxmax,grocer_PRM,grocer_PRV,grocer_prefix_db)
r(1)($+1)='namey'
r('namey')=grocer_names(1)
r(1)($+1) = 'fixed namex'
r('fixed namex')=grocer_names(2)
r(1)($+1) = 'variable namex'
r('variable namex')=grocer_names(3)
r(1)($+1) = 'prests'
r('prests')=grocer_prests
 
if grocer_prests then
   r(1)($+1) = 'bounds'
   r('bounds') = grocer_b
end
 
endfunction
 
 
 
 
 
 
 
 
