function resout=deriv_eq1(reseq,vari)
 
// PURPOSE: provide the derivative of a function with respect
// to a variable x
// ------------------------------------------------------------
// INPUT:
// * reseq = an 'equation' tlist, the result of the breaking
//  down of an equation into elementary bricks
// * vari = a string, the variable the equation is derivated at
// ------------------------------------------------------------
// OUTPUT:
// * resout = a tlist, whose type is equation, with as 2nd
//   argument the type of the opertaion, which can be:
//   - 'operation' (such as '+', '*', '.*' , ...);
//     in that case there are the following fields:
//     . 'lhs': an 'equation' tlist relative to the left hand
//        side of the operation
//     . 'operator': the operator (!)
//     . 'rhs': an 'equation' tlist relative to the right hand
//        side of the operation
//   - 'unary' (when a unary '-', as opposed to the operator,
//     has been found
//     in that case there is the following field:
//     . 'rhs': the equation' tlist relative to the expression
//       after the '-'
//   - 'parentheses'
//     in that case there is the following field:
//     . 'exp': the equation' tlist relative to expression
//       between the parentheses
//   - 'function'
//     in that case there is the following field:
//     . 'input': the 'equation' tlist relative to the input of
//       the function
//   - 'scalar'
//     in that case there is the following field:
//     . 'val': a string, the value of the number that
//       represents the equation
//   - 'variable'
//     in that case there is the following field:
//     . 'name vari': a string, the name of the found variable,
//        assessed as such if it is a matrix or a ts
//   - 'unknown'
//     in that case there is the following field:
//     . 'name unknown': a string, the name of the found
//        unknown object
// ------------------------------------------------------------
// NOTES:
// * this is a recursive function; there is an higher level
//   function deriv_eq that provides simplifications on
//   the structure and transforms the tlist into the text of
//   an equation
// * there is global variable "unknown", whic is set to %t
//   if the euqation is found to contain an unknown object:
//   this can be used, as in nls, to check if you have to use
//   to a numercial derivative instead of an analytical one
// ------------------------------------------------------------
// Copyright: Eric Dubois 2011-2015
// http://grocer.toolbox.free.fr/grocer.html
 
 
if isempty(reseq) then
   warning('equation tlist entry is empty')
   resout=[]
   return
end
 
select reseq('type')
 
case 'parentheses' then
   // simple case: derivate inside the parentheses
   resout=tlist(['equation';'type';'exp'],'parentheses',deriv_eq1(reseq('exp')))
 
case 'unary' then
   // derivate after the unary
   deriv_rhs=deriv_eq1(reseq('rhs'))
   if deriv_rhs('type') == 'scalar' then
      if deriv_rhs('val') == '0' then
      // the derivative is 0 ==> transform -0 into 0
         resout=tlist(['equation';'type';'val'],'scalar','0')
      else
      // keep the derivative
         resout=tlist(['equation';'type';'operator';'rhs'],'unary',reseq('operator'),deriv_eq1(reseq('rhs')))
      end
   else
      resout=tlist(['equation';'type';'operator';'rhs'],'unary',reseq('operator'),deriv_eq1(reseq('rhs')))
   end
 
case 'function' then
   resout=deriv_func(reseq)
 
case 'variable' then
   varif=reseq('name variable')
   if strsubst(varif,' ','') == strsubst(vari,' ','') then
      // the variable is the one used for derivation; the
      // derivative is therefore 1
      resout=tlist(['equation';'type';'val'],'scalar','1')
   else
      // the variable is not the one used for derivation; the
      // derivative is therefore 0
      resout=tlist(['equation';'type';'val'],'scalar','0')
   end
 
case 'operation' then
   lhs_zero=%f
   rhs_zero=%f
   reseq_rhs=reseq('rhs')
   reseq_lhs=reseq('lhs')
 
   // calculates the derivatives of the rhs of the operation
   deriv_rhs=deriv_eq1(reseq_rhs,vari)
   deriv_rhsd=deriv_rhs
   deriv_rhs_typ=deriv_rhs('type')
   while or(deriv_rhs_typ == ['parentheses';'unary']) then
      if deriv_rhs_typ == 'parentheses' then
         deriv_rhsd=deriv_rhsd('exp')
      else
         deriv_rhsd=deriv_rhsd('rhs')
      end
      deriv_rhs_typ=deriv_rhsd('type')
   end
 
   if deriv_rhs_typ == 'scalar' then
      if deriv_rhsd('val') == '0' then
      // the derivative of the rhs of the operation with respect to the
      // variable is 0
         rhs_zero=%t
         deriv_rhs=deriv_rhsd
      end
   end
 
   // calculates the derivatives of the lhs of the operation
   deriv_lhs=deriv_eq1(reseq_lhs,vari)
   deriv_lhs_typ=deriv_lhs('type')
   deriv_lhsd=deriv_lhs
   while or(deriv_lhs_typ == ['parentheses';'unary']) then
      if deriv_lhs_typ == 'parentheses' then
         deriv_lhsd=deriv_lhsd('exp')
      else
         deriv_lhsd=deriv_lhsd('rhs')
      end
      deriv_lhs_typ=deriv_lhsd('type')
   end
   if deriv_lhs_typ == 'scalar' then
      if deriv_lhsd('val') == '0' then
      // the derivative of the lhs of the operation with respect to the
      // variable is 0
         lhs_zero=%t
         deriv_lhs=deriv_lhsd
      end
   end
   select stripblanks(reseq('operator'))
 
   case '+' then
 
      if lhs_zero then
         // the derivative is only made with the rhs derivative
         resout=deriv_rhs
      elseif rhs_zero then
         // the derivative is only made with the lhs derivative
         resout=deriv_lhs
      else
         // add the 2 derivatives
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 deriv_lhs,'+',deriv_rhs)
      end
 
   case '-' then
 
      if rhs_zero then
         // the derivative is only made with the lhs derivative
         resout=deriv_lhs
      elseif lhs_zero then
         // the derivative is only made with the rhs derivative:
         // transform the operator '-' into a unary '-'
         resout=tlist(['equation';'type';'operator';'rhs'],'unary',...
                 '-',deriv_rhs)
      else
         // take the difference of the 2 derivatives
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 deriv_lhs,'-',deriv_rhs)
      end
 
   case '*' then
 
      if lhs_zero  & rhs_zero then
         // the derivative is the sum of two 0s; keep one
         resout=deriv_rhs
 
      elseif rhs_zero then
         // the derivative of the rhs is 0; the derivative of the product is therefore
         // the product of the derivative of the lhs by the rhs
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,'*',reseq_rhs)
 
      elseif lhs_zero then
         // the derivative of the lhs is 0; the derivative of the product is therefore
         // the product of the lhs by the derivative of the rhs
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,'*',deriv_rhs)
 
      else
         // the derivative of the sum of the product of the derivative of the lhs by the rhs
         // and of the product of the lhs by the derivative of the rhs
         resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 deriv_lhs,'*',reseq_rhs),'+',...
                 tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 reseq_lhs,'*',deriv_rhs)))
 
      end
 
   case '.*' then
   // same cases as with '*'
      if lhs_zero  & rhs_zero then
         resout=deriv_rhs
 
      elseif rhs_zero then
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,' .*',reseq_rhs)
 
      elseif lhs_zero then
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,' .*',deriv_rhs)
 
      else
         resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 deriv_lhs,' .*',reseq_rhs),'+',...
                 tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                 reseq_lhs,' .*',deriv_rhs)))
 
      end
 
   case '/' then
 
      if lhs_zero & rhs_zero then
         // the quotient does not depend on the variable: the result is 0
         resout=deriv_lhs
 
      elseif rhs_zero then
         // only the numerator depends on the variable; the derivative of the quotient
         // is therefore
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,'/',reseq_rhs)
 
      elseif lhs_zero then
         while reseq_rhs('type') == 'parentheses'
            reseq_rhs=reseq_rhs('exp')
         end
         true_exp=%f
         if reseq_rhs('type') == 'operation' then
             if or(stripblanks(reseq('operator')) == ['^';'.^']) then
                true_exp=%t
             end
         end
         if true_exp then
            f=reseq_rhs('lhs')
            expon=reseq_rhs('rhs')
            deriv_f=deriv_eq1(f,vari)
            deriv_exp=deriv_eq1(expon,vari)
            resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   tlist(['equation';'type';'operator';'rhs'],'unary','-',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   reseq,' .*',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   tlist(['equation';'type';'exp'],'parentheses',...
                   deriv_exp),' .*',...
                   tlist(['equation';'type';'name func';'input'],'function',...
                   'log',f)))),...
                   '-',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   reseq_lhs,' .*',...
                   tlist(['equation';'type';'exp'],'parentheses',...
                   deriv_f),' .*',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   expon,' ./',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   f,' .^',...
                   tlist(['equation';'type';'exp'],'parentheses',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   expon,'+',tlist(['equation';'type';'val'],'scalar','1'))))))))
         else
            resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'operator';'rhs'],'unary','-',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_rhs,'*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,'/',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'exp'],'parentheses',reseq_rhs),...
                '^',tlist(['equation';'type';'val'],'scalar','2'))))))
 
          end
 
      else
         resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,'/',reseq_rhs),'-',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_rhs,'*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,'/',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'exp'],'parentheses',reseq_rhs),...
                '^',tlist(['equation';'type';'val'],'scalar','2'))))))
      end
 
   case './' then
 
      if lhs_zero & rhs_zero then
         resout=deriv_lhs
 
      elseif rhs_zero then
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,' ./',reseq_rhs)
 
      elseif lhs_zero then
         while reseq_rhs('type') == 'parentheses'
            reseq_rhs=reseq_rhs('exp')
         end
         true_exp=%f
         if reseq_rhs('type') == 'operation' then
             if or(stripblanks(reseq('operator')) == ['^';'.^']) then
                true_exp=%t
             end
         end
         if true_exp then
            f=reseq_rhs('lhs')
            expon=reseq_rhs('rhs')
            deriv_f=deriv_eq1(f,vari)
            deriv_exp=deriv_eq1(expon,vari)
            resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   tlist(['equation';'type';'operator';'rhs'],'unary','-',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   reseq,' .*',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   tlist(['equation';'type';'exp'],'parentheses',...
                   deriv_exp),' .*',...
                   tlist(['equation';'type';'name func';'input'],'function',...
                   'log',f)))),...
                   '-',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   reseq_lhs,' .*',...
                   tlist(['equation';'type';'exp'],'parentheses',...
                   deriv_f),' .*',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   expon,' ./',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   f,' .^',...
                   tlist(['equation';'type';'exp'],'parentheses',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   expon,'-',tlist(['equation';'type';'val'],'scalar','1'))))))))
         else
            resout=tlist(['equation';'type';'exp'],'parentheses',...
                   tlist(['equation';'type';'operator';'rhs'],'unary','-',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   deriv_rhs,' .*',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   reseq_lhs,' ./',...
                   tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                   tlist(['equation';'type';'exp'],'parentheses',reseq_rhs),...
                   '^',tlist(['equation';'type';'val'],'scalar','2'))))))
         end
      else
         resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,' ./',reseq_rhs),'-',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_rhs,' .*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,' ./',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'exp'],'parentheses',reseq_rhs),...
                '^',tlist(['equation';'type';'val'],'scalar','2'))))))
      end
 
   case '^' then
 
      if lhs_zero then
         resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'name func';'input'],'function',...
                'log',reseq_lhs),'*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_rhs,'*',reseq)))
 
      elseif rhs_zero then
         resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_rhs,'*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,'*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,'^',...
                tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_rhs,'-',tlist(['equation';'type';'val'],'scalar','1')))))))
      else
         resout=tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                tlist(['equation';'type';'name func';'input'],'function',...
                'log',reseq_lhs),' .*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_rhs,'*',reseq))),...
                '+',...
                tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_rhs,'*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,'*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,'^',...
                tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_rhs,'-',tlist(['equation';'type';'val'],'scalar','1'))))))))
      end
 
   case ' .^' then
 
      if lhs_zero then
         resout=deriv_lhs
 
      else
         resout=tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                deriv_lhs,' .*',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_lhs,' .^',...
                tlist(['equation';'type';'exp'],'parentheses',...
                tlist(['equation';'type';'lhs';'operator';'rhs'],'operation',...
                reseq_rhs,'-',tlist(['equation';'type';'val'],'scalar','1'))))))
      end
 
   end
 
case 'scalar' then
   resout=tlist(['equation';'type';'val'],'scalar','0')
 
end
 
endfunction
